"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var validator_1 = require("./validator");
/**
 * Main class to handle Chess match using light-chess.
 *
 * @remarks
 * Based on `Validator` namespace and its functions.
 */
var Chess = /** @class */ (function () {
    /**
     * Default class constructor.
     *
     * @param layout - The layout of the chessboard at the beginning of the match, if ignored the default layout will be `Chess.layout`.
     */
    function Chess(layout) {
        var _this = this;
        if (layout === void 0) { layout = Chess.layout; }
        /**
         * The current player turn.
         */
        this.turn = validator_1.Colors.WHITE;
        /**
         * The winner of the match.
         *
         * @remarks
         * if `null` then the match is not finished.
         */
        this.checkmate = null;
        /**
         * Evaluates and validates a move within the current chessboard.
         *
         * @param movement - The movement to validate.
         * @returns One of the possible value of `Validations` enumeration.
         *
         * @remarks
         * Non-static wrapper for respective `Validator.validate` function.
         */
        this.validate = function (movement) {
            return validator_1.Validator.validate(_this.chessboard, movement);
        };
        /**
         * Calculates every legal move for a selected piece within the current chessboard.
         *
         * @param coordinate - The coordinate of the piece.
         * @returns An array containing every square's coordinate where the piece can be moved.
         *
         * @remarks
         * Non-static wrapper for respective `Validator.legals` function.
         */
        this.legals = function (coordinate) {
            return validator_1.Validator.legals(_this.chessboard, coordinate);
        };
        /**
         * Calculates every legal move for a selected player within the current chessboard.
         *
         * @param color - The color of the player to calculate the possible moves. If ignored the default player will be the current turn one.
         * @param check - The default value is true. If it is set to false, all the check control will be disabled.
         * @returns An array containing every possible movement.
         *
         * @remarks
         * Non-static wrapper for respective `Validator.possibilities` function.
         */
        this.possibilities = function (color, check) {
            if (color === void 0) { color = _this.turn; }
            if (check === void 0) { check = true; }
            return validator_1.Validator.possibilities(_this.chessboard, color, check);
        };
        /**
         * Evaluates if the selected player is on check.
         *
         * @param color - The color of the player to evaluate. If ignored the default player will be the current turn one.
         * @returns `true`: The player is on check, `false`: The player is not on check.
         *
         * @remarks
         * Non-static wrapper for respective `Validator.check` function.
         */
        this.check = function (color) {
            if (color === void 0) { color = _this.turn; }
            return validator_1.Validator.check(_this.chessboard, color);
        };
        /**
         * Creates an ASCII string which represents the current chessboard layout.
         *
         * @returns The created ASCII string.
         *
         * @remarks
         * Non-static wrapper for respective `Chess.ascii` static method.
         */
        this.ascii = function () {
            return Chess.ascii(_this.chessboard);
        };
        this.chessboard = layout;
    }
    /**
     * Executes a move for the current player.
     *
     * @param movement - The movement that will be executed.
     * @returns One of the possible value of `Validations` enumeration.
     */
    Chess.prototype.move = function (movement) {
        if (this.checkmate)
            return validator_1.Validations.MATCH_FINISHED;
        var initial = movement.initial, final = movement.final;
        if (this.chessboard[initial.row][initial.column].color != this.turn)
            return validator_1.Validations.ILLEGAL_MOVE;
        var result = this.validate(movement);
        if (result == validator_1.Validations.LEGAL_MOVE) {
            this.chessboard[final.row][final.column] = this.chessboard[initial.row][initial.column];
            this.chessboard[initial.row][initial.column] = validator_1.Pieces.Empty;
            var moves = this.possibilities(this.turn ? validator_1.Colors.WHITE : validator_1.Colors.BLACK);
            if (moves.length == 0) {
                this.checkmate = this.turn;
                return validator_1.Validations.CHECKMATE;
            }
            this.turn = this.turn ? validator_1.Colors.WHITE : validator_1.Colors.BLACK;
            return result;
        }
        return result;
    };
    /**
     * Converts a string format coordinate to the respective `Coordinate` object.
     *
     * @param square - The selected square of the chessboard expressed in string format (example: `e2`, `a5`, `h6`).
     * @returns The respective `Coordinate` object, if the string format is not valid the function will return `null`.
     */
    Chess.decode = function (square) {
        var letters = ["A", "B", "C", "D", "E", "F", "G", "H"];
        var numbers = ["1", "2", "3", "4", "5", "6", "7", "8"];
        if (square == null || square.length > 2)
            return null;
        var X = square[0].toUpperCase(), Y = square[1];
        if (letters.indexOf(X) < 0 || numbers.indexOf(Y) < 0)
            return null;
        var coordinate = {
            row: numbers.indexOf(Y),
            column: letters.indexOf(X)
        };
        return coordinate;
    };
    /**
     * Creates an ASCII string which represents the current chessboard layout.
     *
     * @param chessboard - The chessboard layout to represent.
     * @returns The created ASCII string.
     */
    Chess.ascii = function (chessboard) {
        var emoji = function (piece) {
            switch (piece) {
                case validator_1.Pieces.White.Pawn: return "♙";
                case validator_1.Pieces.White.Bishop: return "♗";
                case validator_1.Pieces.White.Knight: return "♘";
                case validator_1.Pieces.White.Rook: return "♖";
                case validator_1.Pieces.White.Queen: return "♕";
                case validator_1.Pieces.White.King: return "♔";
                case validator_1.Pieces.Black.Pawn: return "♟";
                case validator_1.Pieces.Black.Bishop: return "♝";
                case validator_1.Pieces.Black.Knight: return "♞";
                case validator_1.Pieces.Black.Rook: return "♜";
                case validator_1.Pieces.Black.Queen: return "♛";
                case validator_1.Pieces.Black.King: return "♚";
                case validator_1.Pieces.Empty: return " ";
            }
        };
        var ascii = "";
        for (var row = chessboard.length - 1; row >= 0; row--) {
            var append = (row + 1) + " |";
            for (var column in chessboard[row])
                append = append + emoji(chessboard[row][column]) + " |";
            ascii = ascii + append + "\n";
        }
        ascii = ascii + ("   A  B  C  D  E  F  G  H");
        return ascii;
    };
    /**
     * The default layout for the chessboard.
     */
    Chess.layout = [
        [validator_1.Pieces.White.Rook, validator_1.Pieces.White.Knight, validator_1.Pieces.White.Bishop, validator_1.Pieces.White.Queen, validator_1.Pieces.White.King, validator_1.Pieces.White.Bishop, validator_1.Pieces.White.Knight, validator_1.Pieces.White.Rook],
        [validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn],
        [validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty],
        [validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty],
        [validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty],
        [validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty],
        [validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn],
        [validator_1.Pieces.Black.Rook, validator_1.Pieces.Black.Knight, validator_1.Pieces.Black.Bishop, validator_1.Pieces.Black.Queen, validator_1.Pieces.Black.King, validator_1.Pieces.Black.Bishop, validator_1.Pieces.Black.Knight, validator_1.Pieces.Black.Rook],
    ];
    return Chess;
}());
exports.Chess = Chess;
//# sourceMappingURL=chess.js.map